# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Ds Log Manager Service (2.8.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class LogAppMessageDeclaration(Model):
    """Log app message declaration (log.AppMessageDeclaration)

    Properties:
        attributes: (Attributes) REQUIRED List[str]

        code: (Code) REQUIRED str

        code_name: (CodeName) REQUIRED str

        section: (Section) REQUIRED str

        service: (Service) REQUIRED str

        text: (Text) REQUIRED str
    """

    # region fields

    attributes: List[str]  # REQUIRED
    code: str  # REQUIRED
    code_name: str  # REQUIRED
    section: str  # REQUIRED
    service: str  # REQUIRED
    text: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_attributes(self, value: List[str]) -> LogAppMessageDeclaration:
        self.attributes = value
        return self

    def with_code(self, value: str) -> LogAppMessageDeclaration:
        self.code = value
        return self

    def with_code_name(self, value: str) -> LogAppMessageDeclaration:
        self.code_name = value
        return self

    def with_section(self, value: str) -> LogAppMessageDeclaration:
        self.section = value
        return self

    def with_service(self, value: str) -> LogAppMessageDeclaration:
        self.service = value
        return self

    def with_text(self, value: str) -> LogAppMessageDeclaration:
        self.text = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "attributes"):
            result["Attributes"] = [str(i0) for i0 in self.attributes]
        elif include_empty:
            result["Attributes"] = []
        if hasattr(self, "code"):
            result["Code"] = str(self.code)
        elif include_empty:
            result["Code"] = ""
        if hasattr(self, "code_name"):
            result["CodeName"] = str(self.code_name)
        elif include_empty:
            result["CodeName"] = ""
        if hasattr(self, "section"):
            result["Section"] = str(self.section)
        elif include_empty:
            result["Section"] = ""
        if hasattr(self, "service"):
            result["Service"] = str(self.service)
        elif include_empty:
            result["Service"] = ""
        if hasattr(self, "text"):
            result["Text"] = str(self.text)
        elif include_empty:
            result["Text"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        attributes: List[str],
        code: str,
        code_name: str,
        section: str,
        service: str,
        text: str,
    ) -> LogAppMessageDeclaration:
        instance = cls()
        instance.attributes = attributes
        instance.code = code
        instance.code_name = code_name
        instance.section = section
        instance.service = service
        instance.text = text
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> LogAppMessageDeclaration:
        instance = cls()
        if not dict_:
            return instance
        if "Attributes" in dict_ and dict_["Attributes"] is not None:
            instance.attributes = [str(i0) for i0 in dict_["Attributes"]]
        elif include_empty:
            instance.attributes = []
        if "Code" in dict_ and dict_["Code"] is not None:
            instance.code = str(dict_["Code"])
        elif include_empty:
            instance.code = ""
        if "CodeName" in dict_ and dict_["CodeName"] is not None:
            instance.code_name = str(dict_["CodeName"])
        elif include_empty:
            instance.code_name = ""
        if "Section" in dict_ and dict_["Section"] is not None:
            instance.section = str(dict_["Section"])
        elif include_empty:
            instance.section = ""
        if "Service" in dict_ and dict_["Service"] is not None:
            instance.service = str(dict_["Service"])
        elif include_empty:
            instance.service = ""
        if "Text" in dict_ and dict_["Text"] is not None:
            instance.text = str(dict_["Text"])
        elif include_empty:
            instance.text = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, LogAppMessageDeclaration]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[LogAppMessageDeclaration]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        LogAppMessageDeclaration,
        List[LogAppMessageDeclaration],
        Dict[Any, LogAppMessageDeclaration],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "Attributes": "attributes",
            "Code": "code",
            "CodeName": "code_name",
            "Section": "section",
            "Service": "service",
            "Text": "text",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "Attributes": True,
            "Code": True,
            "CodeName": True,
            "Section": True,
            "Service": True,
            "Text": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Platform Service."""

__version__ = "4.18.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .get_statistic import GetStatistic
from .get_statistic import (
    ActionEnum as GetStatisticActionEnum,
    ItemTypeEnum as GetStatisticItemTypeEnum,
    TypeEnum as GetStatisticTypeEnum,
)
from .publish_all import PublishAll
from .publish_selected import PublishSelected
from .query_changes import QueryChanges
from .query_changes import (
    ActionEnum as QueryChangesActionEnum,
    ItemTypeEnum as QueryChangesItemTypeEnum,
    SortByEnum as QueryChangesSortByEnum,
    StatusEnum as QueryChangesStatusEnum,
    TypeEnum as QueryChangesTypeEnum,
)
from .select_all_records import SelectAllRecords
from .select_record import SelectRecord
from .unselect_all_records import UnselectAllRecords
from .unselect_record import UnselectRecord



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Seasonpass Service (1.17.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class UserSeasonSummary(Model):
    """A DTO object for user season summary. (UserSeasonSummary)

    Properties:
        namespace: (namespace) REQUIRED str

        user_id: (userId) REQUIRED str

        cleared: (cleared) OPTIONAL bool

        created_at: (createdAt) OPTIONAL str

        current_exp: (currentExp) OPTIONAL int

        current_tier_index: (currentTierIndex) OPTIONAL int

        enrolled_at: (enrolledAt) OPTIONAL str

        enrolled_passes: (enrolledPasses) OPTIONAL List[str]

        id_: (id) OPTIONAL str

        last_tier_index: (lastTierIndex) OPTIONAL int

        required_exp: (requiredExp) OPTIONAL int

        season_id: (seasonId) OPTIONAL str

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    namespace: str  # REQUIRED
    user_id: str  # REQUIRED
    cleared: bool  # OPTIONAL
    created_at: str  # OPTIONAL
    current_exp: int  # OPTIONAL
    current_tier_index: int  # OPTIONAL
    enrolled_at: str  # OPTIONAL
    enrolled_passes: List[str]  # OPTIONAL
    id_: str  # OPTIONAL
    last_tier_index: int  # OPTIONAL
    required_exp: int  # OPTIONAL
    season_id: str  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_namespace(self, value: str) -> UserSeasonSummary:
        self.namespace = value
        return self

    def with_user_id(self, value: str) -> UserSeasonSummary:
        self.user_id = value
        return self

    def with_cleared(self, value: bool) -> UserSeasonSummary:
        self.cleared = value
        return self

    def with_created_at(self, value: str) -> UserSeasonSummary:
        self.created_at = value
        return self

    def with_current_exp(self, value: int) -> UserSeasonSummary:
        self.current_exp = value
        return self

    def with_current_tier_index(self, value: int) -> UserSeasonSummary:
        self.current_tier_index = value
        return self

    def with_enrolled_at(self, value: str) -> UserSeasonSummary:
        self.enrolled_at = value
        return self

    def with_enrolled_passes(self, value: List[str]) -> UserSeasonSummary:
        self.enrolled_passes = value
        return self

    def with_id(self, value: str) -> UserSeasonSummary:
        self.id_ = value
        return self

    def with_last_tier_index(self, value: int) -> UserSeasonSummary:
        self.last_tier_index = value
        return self

    def with_required_exp(self, value: int) -> UserSeasonSummary:
        self.required_exp = value
        return self

    def with_season_id(self, value: str) -> UserSeasonSummary:
        self.season_id = value
        return self

    def with_updated_at(self, value: str) -> UserSeasonSummary:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "cleared"):
            result["cleared"] = bool(self.cleared)
        elif include_empty:
            result["cleared"] = False
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "current_exp"):
            result["currentExp"] = int(self.current_exp)
        elif include_empty:
            result["currentExp"] = 0
        if hasattr(self, "current_tier_index"):
            result["currentTierIndex"] = int(self.current_tier_index)
        elif include_empty:
            result["currentTierIndex"] = 0
        if hasattr(self, "enrolled_at"):
            result["enrolledAt"] = str(self.enrolled_at)
        elif include_empty:
            result["enrolledAt"] = ""
        if hasattr(self, "enrolled_passes"):
            result["enrolledPasses"] = [str(i0) for i0 in self.enrolled_passes]
        elif include_empty:
            result["enrolledPasses"] = []
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "last_tier_index"):
            result["lastTierIndex"] = int(self.last_tier_index)
        elif include_empty:
            result["lastTierIndex"] = 0
        if hasattr(self, "required_exp"):
            result["requiredExp"] = int(self.required_exp)
        elif include_empty:
            result["requiredExp"] = 0
        if hasattr(self, "season_id"):
            result["seasonId"] = str(self.season_id)
        elif include_empty:
            result["seasonId"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        user_id: str,
        cleared: Optional[bool] = None,
        created_at: Optional[str] = None,
        current_exp: Optional[int] = None,
        current_tier_index: Optional[int] = None,
        enrolled_at: Optional[str] = None,
        enrolled_passes: Optional[List[str]] = None,
        id_: Optional[str] = None,
        last_tier_index: Optional[int] = None,
        required_exp: Optional[int] = None,
        season_id: Optional[str] = None,
        updated_at: Optional[str] = None,
    ) -> UserSeasonSummary:
        instance = cls()
        instance.namespace = namespace
        instance.user_id = user_id
        if cleared is not None:
            instance.cleared = cleared
        if created_at is not None:
            instance.created_at = created_at
        if current_exp is not None:
            instance.current_exp = current_exp
        if current_tier_index is not None:
            instance.current_tier_index = current_tier_index
        if enrolled_at is not None:
            instance.enrolled_at = enrolled_at
        if enrolled_passes is not None:
            instance.enrolled_passes = enrolled_passes
        if id_ is not None:
            instance.id_ = id_
        if last_tier_index is not None:
            instance.last_tier_index = last_tier_index
        if required_exp is not None:
            instance.required_exp = required_exp
        if season_id is not None:
            instance.season_id = season_id
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UserSeasonSummary:
        instance = cls()
        if not dict_:
            return instance
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "cleared" in dict_ and dict_["cleared"] is not None:
            instance.cleared = bool(dict_["cleared"])
        elif include_empty:
            instance.cleared = False
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "currentExp" in dict_ and dict_["currentExp"] is not None:
            instance.current_exp = int(dict_["currentExp"])
        elif include_empty:
            instance.current_exp = 0
        if "currentTierIndex" in dict_ and dict_["currentTierIndex"] is not None:
            instance.current_tier_index = int(dict_["currentTierIndex"])
        elif include_empty:
            instance.current_tier_index = 0
        if "enrolledAt" in dict_ and dict_["enrolledAt"] is not None:
            instance.enrolled_at = str(dict_["enrolledAt"])
        elif include_empty:
            instance.enrolled_at = ""
        if "enrolledPasses" in dict_ and dict_["enrolledPasses"] is not None:
            instance.enrolled_passes = [str(i0) for i0 in dict_["enrolledPasses"]]
        elif include_empty:
            instance.enrolled_passes = []
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "lastTierIndex" in dict_ and dict_["lastTierIndex"] is not None:
            instance.last_tier_index = int(dict_["lastTierIndex"])
        elif include_empty:
            instance.last_tier_index = 0
        if "requiredExp" in dict_ and dict_["requiredExp"] is not None:
            instance.required_exp = int(dict_["requiredExp"])
        elif include_empty:
            instance.required_exp = 0
        if "seasonId" in dict_ and dict_["seasonId"] is not None:
            instance.season_id = str(dict_["seasonId"])
        elif include_empty:
            instance.season_id = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, UserSeasonSummary]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[UserSeasonSummary]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        UserSeasonSummary, List[UserSeasonSummary], Dict[Any, UserSeasonSummary]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "userId": "user_id",
            "cleared": "cleared",
            "createdAt": "created_at",
            "currentExp": "current_exp",
            "currentTierIndex": "current_tier_index",
            "enrolledAt": "enrolled_at",
            "enrolledPasses": "enrolled_passes",
            "id": "id_",
            "lastTierIndex": "last_tier_index",
            "requiredExp": "required_exp",
            "seasonId": "season_id",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "userId": True,
            "cleared": False,
            "createdAt": False,
            "currentExp": False,
            "currentTierIndex": False,
            "enrolledAt": False,
            "enrolledPasses": False,
            "id": False,
            "lastTierIndex": False,
            "requiredExp": False,
            "seasonId": False,
            "updatedAt": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from re import Pattern
from typing import Any, Dict, List, Tuple, Union

from ._validators import validate_field


class Model:
    def __str__(self):
        return str(self.to_dict())

    def is_valid(
        self,
        check_required: bool = True,
        check_enum: bool = True,
        check_pattern: bool = True,
        recursive_check: bool = True,
        **kwargs
    ) -> Tuple[bool, Union[None, str]]:
        field_info = self.get_field_info()
        required_map = self.get_required_map() if check_required else None
        enum_map = self.get_enum_map() if check_enum else None
        pattern_map = self.get_pattern_map() if check_pattern else None
        for name in field_info:
            is_valid, error = validate_field(
                obj=self,
                name=name,
                alias_map=field_info,
                required_map=required_map,
                enum_map=enum_map,
                pattern_map=pattern_map,
            )
            if not is_valid:
                return False, error

            if recursive_check:
                kw = {
                    "check_required": check_required,
                    "check_enum": check_enum,
                    "check_pattern": check_pattern,
                    "recursive_check": recursive_check,
                }
                alias = field_info[name]
                value = getattr(self, alias) if hasattr(self, alias) else None
                if value:
                    if isinstance(value, list):
                        for x in value:
                            if not isinstance(x, Model):
                                break
                            is_valid, error = x.is_valid(**kw)
                            if not is_valid:
                                return False, error
                    else:
                        if isinstance(value, Model):
                            is_valid, error = value.is_valid(**kw)
                            if not is_valid:
                                return False, error
        return True, None

    # region overrideable members

    # noinspection PyMethodMayBeStatic
    def to_dict(self, include_empty: bool = False) -> dict:
        # pylint: disable=no-self-use, unused-argument
        return {}

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        raise NotImplementedError

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {}

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {}

    @staticmethod
    def get_pattern_map() -> Dict[str, Pattern]:
        return {}

    # endregion overrideable members



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import CurrencyConfig
from ..models import CurrencyCreate
from ..models import CurrencyInfo
from ..models import CurrencySummary
from ..models import CurrencyUpdate
from ..models import ErrorEntity
from ..models import ValidationErrorEntity

from ..operations.currency import CreateCurrency
from ..operations.currency import DeleteCurrency
from ..operations.currency import GetCurrencyConfig
from ..operations.currency import GetCurrencySummary
from ..operations.currency import ListCurrencies
from ..operations.currency import ListCurrenciesCurrencyTypeEnum
from ..operations.currency import PublicListCurrencies
from ..operations.currency import PublicListCurrenciesCurrencyTypeEnum
from ..operations.currency import UpdateCurrency
from ..models import CurrencyCreateCurrencyTypeEnum
from ..models import CurrencyInfoCurrencyTypeEnum
from ..models import CurrencySummaryCurrencyTypeEnum


@same_doc_as(CreateCurrency)
def create_currency(
    body: Optional[CurrencyCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCurrency.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCurrency)
async def create_currency_async(
    body: Optional[CurrencyCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCurrency.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteCurrency)
def delete_currency(
    currency_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteCurrency.create(
        currency_code=currency_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteCurrency)
async def delete_currency_async(
    currency_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteCurrency.create(
        currency_code=currency_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCurrencyConfig)
def get_currency_config(
    currency_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCurrencyConfig.create(
        currency_code=currency_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCurrencyConfig)
async def get_currency_config_async(
    currency_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCurrencyConfig.create(
        currency_code=currency_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCurrencySummary)
def get_currency_summary(
    currency_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCurrencySummary.create(
        currency_code=currency_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCurrencySummary)
async def get_currency_summary_async(
    currency_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCurrencySummary.create(
        currency_code=currency_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListCurrencies)
def list_currencies(
    currency_type: Optional[Union[str, ListCurrenciesCurrencyTypeEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListCurrencies.create(
        currency_type=currency_type,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListCurrencies)
async def list_currencies_async(
    currency_type: Optional[Union[str, ListCurrenciesCurrencyTypeEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListCurrencies.create(
        currency_type=currency_type,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicListCurrencies)
def public_list_currencies(
    currency_type: Optional[Union[str, PublicListCurrenciesCurrencyTypeEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListCurrencies.create(
        currency_type=currency_type,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicListCurrencies)
async def public_list_currencies_async(
    currency_type: Optional[Union[str, PublicListCurrenciesCurrencyTypeEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListCurrencies.create(
        currency_type=currency_type,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateCurrency)
def update_currency(
    currency_code: str,
    body: Optional[CurrencyUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCurrency.create(
        currency_code=currency_code,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateCurrency)
async def update_currency_async(
    currency_code: str,
    body: Optional[CurrencyUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCurrency.create(
        currency_code=currency_code,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.18.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ConditionMatchResult(Model):
    """Condition match result (ConditionMatchResult)

    Properties:
        matched: (matched) OPTIONAL bool

        matched_conditions: (matchedConditions) OPTIONAL List[Dict[str, Any]]

        not_match_reason: (notMatchReason) OPTIONAL str
    """

    # region fields

    matched: bool  # OPTIONAL
    matched_conditions: List[Dict[str, Any]]  # OPTIONAL
    not_match_reason: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_matched(self, value: bool) -> ConditionMatchResult:
        self.matched = value
        return self

    def with_matched_conditions(
        self, value: List[Dict[str, Any]]
    ) -> ConditionMatchResult:
        self.matched_conditions = value
        return self

    def with_not_match_reason(self, value: str) -> ConditionMatchResult:
        self.not_match_reason = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "matched"):
            result["matched"] = bool(self.matched)
        elif include_empty:
            result["matched"] = False
        if hasattr(self, "matched_conditions"):
            result["matchedConditions"] = [
                {str(k1): v1 for k1, v1 in i0.items()} for i0 in self.matched_conditions
            ]
        elif include_empty:
            result["matchedConditions"] = []
        if hasattr(self, "not_match_reason"):
            result["notMatchReason"] = str(self.not_match_reason)
        elif include_empty:
            result["notMatchReason"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        matched: Optional[bool] = None,
        matched_conditions: Optional[List[Dict[str, Any]]] = None,
        not_match_reason: Optional[str] = None,
    ) -> ConditionMatchResult:
        instance = cls()
        if matched is not None:
            instance.matched = matched
        if matched_conditions is not None:
            instance.matched_conditions = matched_conditions
        if not_match_reason is not None:
            instance.not_match_reason = not_match_reason
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ConditionMatchResult:
        instance = cls()
        if not dict_:
            return instance
        if "matched" in dict_ and dict_["matched"] is not None:
            instance.matched = bool(dict_["matched"])
        elif include_empty:
            instance.matched = False
        if "matchedConditions" in dict_ and dict_["matchedConditions"] is not None:
            instance.matched_conditions = [
                {str(k1): v1 for k1, v1 in i0.items()}
                for i0 in dict_["matchedConditions"]
            ]
        elif include_empty:
            instance.matched_conditions = []
        if "notMatchReason" in dict_ and dict_["notMatchReason"] is not None:
            instance.not_match_reason = str(dict_["notMatchReason"])
        elif include_empty:
            instance.not_match_reason = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ConditionMatchResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ConditionMatchResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ConditionMatchResult,
        List[ConditionMatchResult],
        Dict[Any, ConditionMatchResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "matched": "matched",
            "matchedConditions": "matched_conditions",
            "notMatchReason": "not_match_reason",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "matched": False,
            "matchedConditions": False,
            "notMatchReason": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import BulkOperationResult
from ..models import CampaignCreate
from ..models import CampaignDynamicInfo
from ..models import CampaignInfo
from ..models import CampaignPagingSlicedResult
from ..models import CampaignUpdate
from ..models import CodeCreate
from ..models import CodeCreateResult
from ..models import CodeInfo
from ..models import CodeInfoPagingSlicedResult
from ..models import ErrorEntity
from ..models import RedeemHistoryPagingSlicedResult
from ..models import RedeemRequest
from ..models import RedeemResult
from ..models import ValidationErrorEntity

from ..operations.campaign import ApplyUserRedemption
from ..operations.campaign import BulkDisableCodes
from ..operations.campaign import BulkEnableCodes
from ..operations.campaign import CreateCampaign
from ..operations.campaign import CreateCodes
from ..operations.campaign import DisableCode
from ..operations.campaign import Download
from ..operations.campaign import EnableCode
from ..operations.campaign import GetCampaign
from ..operations.campaign import GetCampaignDynamic
from ..operations.campaign import GetCode
from ..operations.campaign import QueryCampaigns
from ..operations.campaign import QueryCodes
from ..operations.campaign import QueryRedeemHistory
from ..operations.campaign import UpdateCampaign
from ..models import (
    CampaignCreateRedeemTypeEnum,
    CampaignCreateStatusEnum,
    CampaignCreateTypeEnum,
)
from ..models import (
    CampaignInfoRedeemTypeEnum,
    CampaignInfoStatusEnum,
    CampaignInfoTypeEnum,
)
from ..models import CampaignUpdateRedeemTypeEnum, CampaignUpdateStatusEnum
from ..models import CodeInfoRedeemTypeEnum, CodeInfoStatusEnum, CodeInfoTypeEnum


@same_doc_as(ApplyUserRedemption)
def apply_user_redemption(
    user_id: str,
    body: Optional[RedeemRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ApplyUserRedemption.create(
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ApplyUserRedemption)
async def apply_user_redemption_async(
    user_id: str,
    body: Optional[RedeemRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ApplyUserRedemption.create(
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkDisableCodes)
def bulk_disable_codes(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkDisableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkDisableCodes)
async def bulk_disable_codes_async(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkDisableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkEnableCodes)
def bulk_enable_codes(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkEnableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkEnableCodes)
async def bulk_enable_codes_async(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkEnableCodes.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateCampaign)
def create_campaign(
    body: Optional[CampaignCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCampaign.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCampaign)
async def create_campaign_async(
    body: Optional[CampaignCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCampaign.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateCodes)
def create_codes(
    campaign_id: str,
    body: Optional[CodeCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCodes.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateCodes)
async def create_codes_async(
    campaign_id: str,
    body: Optional[CodeCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateCodes.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DisableCode)
def disable_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DisableCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DisableCode)
async def disable_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DisableCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(Download)
def download(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Download.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Download)
async def download_async(
    campaign_id: str,
    batch_no: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Download.create(
        campaign_id=campaign_id,
        batch_no=batch_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(EnableCode)
def enable_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = EnableCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(EnableCode)
async def enable_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = EnableCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCampaign)
def get_campaign(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaign.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCampaign)
async def get_campaign_async(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaign.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCampaignDynamic)
def get_campaign_dynamic(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaignDynamic.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCampaignDynamic)
async def get_campaign_dynamic_async(
    campaign_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCampaignDynamic.create(
        campaign_id=campaign_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCode)
def get_code(
    code: str,
    redeemable: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCode.create(
        code=code,
        redeemable=redeemable,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCode)
async def get_code_async(
    code: str,
    redeemable: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetCode.create(
        code=code,
        redeemable=redeemable,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCampaigns)
def query_campaigns(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    tag: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaigns.create(
        limit=limit,
        name=name,
        offset=offset,
        tag=tag,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCampaigns)
async def query_campaigns_async(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    tag: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCampaigns.create(
        limit=limit,
        name=name,
        offset=offset,
        tag=tag,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryCodes)
def query_codes(
    campaign_id: str,
    active_only: Optional[bool] = None,
    batch_no: Optional[int] = None,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCodes.create(
        campaign_id=campaign_id,
        active_only=active_only,
        batch_no=batch_no,
        code=code,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryCodes)
async def query_codes_async(
    campaign_id: str,
    active_only: Optional[bool] = None,
    batch_no: Optional[int] = None,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryCodes.create(
        campaign_id=campaign_id,
        active_only=active_only,
        batch_no=batch_no,
        code=code,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryRedeemHistory)
def query_redeem_history(
    campaign_id: str,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryRedeemHistory.create(
        campaign_id=campaign_id,
        code=code,
        limit=limit,
        offset=offset,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryRedeemHistory)
async def query_redeem_history_async(
    campaign_id: str,
    code: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryRedeemHistory.create(
        campaign_id=campaign_id,
        code=code,
        limit=limit,
        offset=offset,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateCampaign)
def update_campaign(
    campaign_id: str,
    body: Optional[CampaignUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCampaign.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateCampaign)
async def update_campaign_async(
    campaign_id: str,
    body: Optional[CampaignUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateCampaign.create(
        campaign_id=campaign_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.18.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    ACTIVE = "ACTIVE"
    CANCELLED = "CANCELLED"
    EXPIRED = "EXPIRED"
    INIT = "INIT"


class SubscribedByEnum(StrEnum):
    PLATFORM = "PLATFORM"
    USER = "USER"


class SubscriptionSummary(Model):
    """Subscription summary (SubscriptionSummary)

    Properties:
        id_: (id) REQUIRED str

        item_id: (itemId) REQUIRED str

        namespace: (namespace) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        user_id: (userId) REQUIRED str

        current_period_end: (currentPeriodEnd) OPTIONAL str

        current_period_start: (currentPeriodStart) OPTIONAL str

        sku: (sku) OPTIONAL str

        subscribed_by: (subscribedBy) OPTIONAL Union[str, SubscribedByEnum]
    """

    # region fields

    id_: str  # REQUIRED
    item_id: str  # REQUIRED
    namespace: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    user_id: str  # REQUIRED
    current_period_end: str  # OPTIONAL
    current_period_start: str  # OPTIONAL
    sku: str  # OPTIONAL
    subscribed_by: Union[str, SubscribedByEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> SubscriptionSummary:
        self.id_ = value
        return self

    def with_item_id(self, value: str) -> SubscriptionSummary:
        self.item_id = value
        return self

    def with_namespace(self, value: str) -> SubscriptionSummary:
        self.namespace = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> SubscriptionSummary:
        self.status = value
        return self

    def with_user_id(self, value: str) -> SubscriptionSummary:
        self.user_id = value
        return self

    def with_current_period_end(self, value: str) -> SubscriptionSummary:
        self.current_period_end = value
        return self

    def with_current_period_start(self, value: str) -> SubscriptionSummary:
        self.current_period_start = value
        return self

    def with_sku(self, value: str) -> SubscriptionSummary:
        self.sku = value
        return self

    def with_subscribed_by(
        self, value: Union[str, SubscribedByEnum]
    ) -> SubscriptionSummary:
        self.subscribed_by = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "current_period_end"):
            result["currentPeriodEnd"] = str(self.current_period_end)
        elif include_empty:
            result["currentPeriodEnd"] = ""
        if hasattr(self, "current_period_start"):
            result["currentPeriodStart"] = str(self.current_period_start)
        elif include_empty:
            result["currentPeriodStart"] = ""
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "subscribed_by"):
            result["subscribedBy"] = str(self.subscribed_by)
        elif include_empty:
            result["subscribedBy"] = Union[str, SubscribedByEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        item_id: str,
        namespace: str,
        status: Union[str, StatusEnum],
        user_id: str,
        current_period_end: Optional[str] = None,
        current_period_start: Optional[str] = None,
        sku: Optional[str] = None,
        subscribed_by: Optional[Union[str, SubscribedByEnum]] = None,
    ) -> SubscriptionSummary:
        instance = cls()
        instance.id_ = id_
        instance.item_id = item_id
        instance.namespace = namespace
        instance.status = status
        instance.user_id = user_id
        if current_period_end is not None:
            instance.current_period_end = current_period_end
        if current_period_start is not None:
            instance.current_period_start = current_period_start
        if sku is not None:
            instance.sku = sku
        if subscribed_by is not None:
            instance.subscribed_by = subscribed_by
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> SubscriptionSummary:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "currentPeriodEnd" in dict_ and dict_["currentPeriodEnd"] is not None:
            instance.current_period_end = str(dict_["currentPeriodEnd"])
        elif include_empty:
            instance.current_period_end = ""
        if "currentPeriodStart" in dict_ and dict_["currentPeriodStart"] is not None:
            instance.current_period_start = str(dict_["currentPeriodStart"])
        elif include_empty:
            instance.current_period_start = ""
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "subscribedBy" in dict_ and dict_["subscribedBy"] is not None:
            instance.subscribed_by = str(dict_["subscribedBy"])
        elif include_empty:
            instance.subscribed_by = Union[str, SubscribedByEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, SubscriptionSummary]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[SubscriptionSummary]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        SubscriptionSummary, List[SubscriptionSummary], Dict[Any, SubscriptionSummary]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "itemId": "item_id",
            "namespace": "namespace",
            "status": "status",
            "userId": "user_id",
            "currentPeriodEnd": "current_period_end",
            "currentPeriodStart": "current_period_start",
            "sku": "sku",
            "subscribedBy": "subscribed_by",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": True,
            "itemId": True,
            "namespace": True,
            "status": True,
            "userId": True,
            "currentPeriodEnd": False,
            "currentPeriodStart": False,
            "sku": False,
            "subscribedBy": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["ACTIVE", "CANCELLED", "EXPIRED", "INIT"],
            "subscribedBy": ["PLATFORM", "USER"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsPartyData
from ..models import ModelsPartyPUTCustomAttributesRequest
from ..models import ModelsPartyPUTLimitSizeRequest
from ..models import RestapiErrorResponseBody

from ..operations.party import AdminGetPartyDataV1
from ..operations.party import AdminGetUserPartyV1
from ..operations.party import PublicGetPartyDataV1
from ..operations.party import PublicSetPartyLimitV1
from ..operations.party import PublicUpdatePartyAttributesV1


@same_doc_as(AdminGetPartyDataV1)
def admin_get_party_data_v1(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetPartyDataV1)
async def admin_get_party_data_v1_async(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetUserPartyV1)
def admin_get_user_party_v1(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetUserPartyV1.create(
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetUserPartyV1)
async def admin_get_user_party_v1_async(
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetUserPartyV1.create(
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetPartyDataV1)
def public_get_party_data_v1(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetPartyDataV1)
async def public_get_party_data_v1_async(
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPartyDataV1.create(
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicSetPartyLimitV1)
def public_set_party_limit_v1(
    body: ModelsPartyPUTLimitSizeRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicSetPartyLimitV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicSetPartyLimitV1)
async def public_set_party_limit_v1_async(
    body: ModelsPartyPUTLimitSizeRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicSetPartyLimitV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicUpdatePartyAttributesV1)
def public_update_party_attributes_v1(
    body: ModelsPartyPUTCustomAttributesRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicUpdatePartyAttributesV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicUpdatePartyAttributesV1)
async def public_update_party_attributes_v1_async(
    body: ModelsPartyPUTCustomAttributesRequest,
    party_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicUpdatePartyAttributesV1.create(
        body=body,
        party_id=party_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import ModelsEvent
from ..models import ModelsEventResponse

from ..operations.event import GetEventByEventIDHandler
from ..operations.event import GetEventByEventTypeAndEventIDHandler
from ..operations.event import GetEventByEventTypeHandler
from ..operations.event import GetEventByNamespaceHandler
from ..operations.event import GetEventByUserEventIDAndEventTypeHandler
from ..operations.event import GetEventByUserIDAndEventIDHandler
from ..operations.event import GetEventByUserIDAndEventTypeHandler
from ..operations.event import GetEventByUserIDHandler
from ..operations.event import PostEventHandler


@deprecated
@same_doc_as(GetEventByEventIDHandler)
def get_event_by_event_id_handler(
    end_date: str,
    event_id: float,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByEventIDHandler.create(
        end_date=end_date,
        event_id=event_id,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByEventIDHandler)
async def get_event_by_event_id_handler_async(
    end_date: str,
    event_id: float,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByEventIDHandler.create(
        end_date=end_date,
        event_id=event_id,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByEventTypeAndEventIDHandler)
def get_event_by_event_type_and_event_id_handler(
    end_date: str,
    event_id: float,
    event_type: float,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByEventTypeAndEventIDHandler.create(
        end_date=end_date,
        event_id=event_id,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByEventTypeAndEventIDHandler)
async def get_event_by_event_type_and_event_id_handler_async(
    end_date: str,
    event_id: float,
    event_type: float,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByEventTypeAndEventIDHandler.create(
        end_date=end_date,
        event_id=event_id,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByEventTypeHandler)
def get_event_by_event_type_handler(
    end_date: str,
    event_type: float,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByEventTypeHandler.create(
        end_date=end_date,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByEventTypeHandler)
async def get_event_by_event_type_handler_async(
    end_date: str,
    event_type: float,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByEventTypeHandler.create(
        end_date=end_date,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByNamespaceHandler)
def get_event_by_namespace_handler(
    end_date: str,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByNamespaceHandler.create(
        end_date=end_date,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByNamespaceHandler)
async def get_event_by_namespace_handler_async(
    end_date: str,
    page_size: int,
    start_date: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByNamespaceHandler.create(
        end_date=end_date,
        page_size=page_size,
        start_date=start_date,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByUserEventIDAndEventTypeHandler)
def get_event_by_user_event_id_and_event_type_handler(
    end_date: str,
    event_id: float,
    event_type: float,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserEventIDAndEventTypeHandler.create(
        end_date=end_date,
        event_id=event_id,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByUserEventIDAndEventTypeHandler)
async def get_event_by_user_event_id_and_event_type_handler_async(
    end_date: str,
    event_id: float,
    event_type: float,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserEventIDAndEventTypeHandler.create(
        end_date=end_date,
        event_id=event_id,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByUserIDAndEventIDHandler)
def get_event_by_user_id_and_event_id_handler(
    end_date: str,
    event_id: float,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserIDAndEventIDHandler.create(
        end_date=end_date,
        event_id=event_id,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByUserIDAndEventIDHandler)
async def get_event_by_user_id_and_event_id_handler_async(
    end_date: str,
    event_id: float,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserIDAndEventIDHandler.create(
        end_date=end_date,
        event_id=event_id,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByUserIDAndEventTypeHandler)
def get_event_by_user_id_and_event_type_handler(
    end_date: str,
    event_type: float,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserIDAndEventTypeHandler.create(
        end_date=end_date,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByUserIDAndEventTypeHandler)
async def get_event_by_user_id_and_event_type_handler_async(
    end_date: str,
    event_type: float,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserIDAndEventTypeHandler.create(
        end_date=end_date,
        event_type=event_type,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetEventByUserIDHandler)
def get_event_by_user_id_handler(
    end_date: str,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserIDHandler.create(
        end_date=end_date,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetEventByUserIDHandler)
async def get_event_by_user_id_handler_async(
    end_date: str,
    page_size: int,
    start_date: str,
    user_id: str,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetEventByUserIDHandler.create(
        end_date=end_date,
        page_size=page_size,
        start_date=start_date,
        user_id=user_id,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(PostEventHandler)
def post_event_handler(
    body: ModelsEvent,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PostEventHandler.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(PostEventHandler)
async def post_event_handler_async(
    body: ModelsEvent,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PostEventHandler.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import OauthmodelCountryLocationResponse
from ..models import OauthmodelErrorResponse
from ..models import OauthmodelGameTokenCodeResponse
from ..models import OauthmodelTokenResponseV3
from ..models import RestErrorResponse

from ..operations.o_auth2_0_extension import AuthenticationWithPlatformLinkV3
from ..operations.o_auth2_0_extension import GenerateTokenByNewHeadlessAccountV3
from ..operations.o_auth2_0_extension import GetCountryLocationV3
from ..operations.o_auth2_0_extension import Logout
from ..operations.o_auth2_0_extension import PlatformAuthenticationV3
from ..operations.o_auth2_0_extension import RequestGameTokenCodeResponseV3
from ..operations.o_auth2_0_extension import RequestGameTokenResponseV3
from ..operations.o_auth2_0_extension import UserAuthenticationV3


@same_doc_as(AuthenticationWithPlatformLinkV3)
def authentication_with_platform_link_v3(
    client_id: str,
    linking_token: str,
    password: str,
    username: str,
    extend_exp: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = AuthenticationWithPlatformLinkV3.create(
        client_id=client_id,
        linking_token=linking_token,
        password=password,
        username=username,
        extend_exp=extend_exp,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AuthenticationWithPlatformLinkV3)
async def authentication_with_platform_link_v3_async(
    client_id: str,
    linking_token: str,
    password: str,
    username: str,
    extend_exp: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = AuthenticationWithPlatformLinkV3.create(
        client_id=client_id,
        linking_token=linking_token,
        password=password,
        username=username,
        extend_exp=extend_exp,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GenerateTokenByNewHeadlessAccountV3)
def generate_token_by_new_headless_account_v3(
    linking_token: str,
    extend_exp: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = GenerateTokenByNewHeadlessAccountV3.create(
        linking_token=linking_token,
        extend_exp=extend_exp,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GenerateTokenByNewHeadlessAccountV3)
async def generate_token_by_new_headless_account_v3_async(
    linking_token: str,
    extend_exp: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = GenerateTokenByNewHeadlessAccountV3.create(
        linking_token=linking_token,
        extend_exp=extend_exp,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetCountryLocationV3)
def get_country_location_v3(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = GetCountryLocationV3.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetCountryLocationV3)
async def get_country_location_v3_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = GetCountryLocationV3.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(Logout)
def logout(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    request = Logout.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Logout)
async def logout_async(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    request = Logout.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PlatformAuthenticationV3)
def platform_authentication_v3(
    platform_id: str,
    state: str,
    code: Optional[str] = None,
    error: Optional[str] = None,
    openid_assoc_handle: Optional[str] = None,
    openid_claimed_id: Optional[str] = None,
    openid_identity: Optional[str] = None,
    openid_mode: Optional[str] = None,
    openid_ns: Optional[str] = None,
    openid_op_endpoint: Optional[str] = None,
    openid_response_nonce: Optional[str] = None,
    openid_return_to: Optional[str] = None,
    openid_sig: Optional[str] = None,
    openid_signed: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = PlatformAuthenticationV3.create(
        platform_id=platform_id,
        state=state,
        code=code,
        error=error,
        openid_assoc_handle=openid_assoc_handle,
        openid_claimed_id=openid_claimed_id,
        openid_identity=openid_identity,
        openid_mode=openid_mode,
        openid_ns=openid_ns,
        openid_op_endpoint=openid_op_endpoint,
        openid_response_nonce=openid_response_nonce,
        openid_return_to=openid_return_to,
        openid_sig=openid_sig,
        openid_signed=openid_signed,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PlatformAuthenticationV3)
async def platform_authentication_v3_async(
    platform_id: str,
    state: str,
    code: Optional[str] = None,
    error: Optional[str] = None,
    openid_assoc_handle: Optional[str] = None,
    openid_claimed_id: Optional[str] = None,
    openid_identity: Optional[str] = None,
    openid_mode: Optional[str] = None,
    openid_ns: Optional[str] = None,
    openid_op_endpoint: Optional[str] = None,
    openid_response_nonce: Optional[str] = None,
    openid_return_to: Optional[str] = None,
    openid_sig: Optional[str] = None,
    openid_signed: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = PlatformAuthenticationV3.create(
        platform_id=platform_id,
        state=state,
        code=code,
        error=error,
        openid_assoc_handle=openid_assoc_handle,
        openid_claimed_id=openid_claimed_id,
        openid_identity=openid_identity,
        openid_mode=openid_mode,
        openid_ns=openid_ns,
        openid_op_endpoint=openid_op_endpoint,
        openid_response_nonce=openid_response_nonce,
        openid_return_to=openid_return_to,
        openid_sig=openid_sig,
        openid_signed=openid_signed,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RequestGameTokenCodeResponseV3)
def request_game_token_code_response_v3(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RequestGameTokenCodeResponseV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RequestGameTokenCodeResponseV3)
async def request_game_token_code_response_v3_async(
    client_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RequestGameTokenCodeResponseV3.create(
        client_id=client_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RequestGameTokenResponseV3)
def request_game_token_response_v3(
    code: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = RequestGameTokenResponseV3.create(
        code=code,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RequestGameTokenResponseV3)
async def request_game_token_response_v3_async(
    code: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = RequestGameTokenResponseV3.create(
        code=code,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UserAuthenticationV3)
def user_authentication_v3(
    password: str,
    request_id: str,
    user_name: str,
    client_id: Optional[str] = None,
    extend_exp: Optional[bool] = None,
    redirect_uri: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = UserAuthenticationV3.create(
        password=password,
        request_id=request_id,
        user_name=user_name,
        client_id=client_id,
        extend_exp=extend_exp,
        redirect_uri=redirect_uri,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UserAuthenticationV3)
async def user_authentication_v3_async(
    password: str,
    request_id: str,
    user_name: str,
    client_id: Optional[str] = None,
    extend_exp: Optional[bool] = None,
    redirect_uri: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = UserAuthenticationV3.create(
        password=password,
        request_id=request_id,
        user_name=user_name,
        client_id=client_id,
        extend_exp=extend_exp,
        redirect_uri=redirect_uri,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.18.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class PlatformEnum(StrEnum):
    EPICGAMES = "EPICGAMES"
    PSN = "PSN"
    STEAM = "STEAM"
    XBOX = "XBOX"


class PlatformDlcEntry(Model):
    """Platform dlc entry (PlatformDlcEntry)

    Properties:
        platform: (platform) OPTIONAL Union[str, PlatformEnum]

        platform_dlc_id_map: (platformDlcIdMap) OPTIONAL Dict[str, str]
    """

    # region fields

    platform: Union[str, PlatformEnum]  # OPTIONAL
    platform_dlc_id_map: Dict[str, str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_platform(self, value: Union[str, PlatformEnum]) -> PlatformDlcEntry:
        self.platform = value
        return self

    def with_platform_dlc_id_map(self, value: Dict[str, str]) -> PlatformDlcEntry:
        self.platform_dlc_id_map = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = Union[str, PlatformEnum]()
        if hasattr(self, "platform_dlc_id_map"):
            result["platformDlcIdMap"] = {
                str(k0): str(v0) for k0, v0 in self.platform_dlc_id_map.items()
            }
        elif include_empty:
            result["platformDlcIdMap"] = {}
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        platform: Optional[Union[str, PlatformEnum]] = None,
        platform_dlc_id_map: Optional[Dict[str, str]] = None,
    ) -> PlatformDlcEntry:
        instance = cls()
        if platform is not None:
            instance.platform = platform
        if platform_dlc_id_map is not None:
            instance.platform_dlc_id_map = platform_dlc_id_map
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PlatformDlcEntry:
        instance = cls()
        if not dict_:
            return instance
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = Union[str, PlatformEnum]()
        if "platformDlcIdMap" in dict_ and dict_["platformDlcIdMap"] is not None:
            instance.platform_dlc_id_map = {
                str(k0): str(v0) for k0, v0 in dict_["platformDlcIdMap"].items()
            }
        elif include_empty:
            instance.platform_dlc_id_map = {}
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PlatformDlcEntry]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PlatformDlcEntry]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[PlatformDlcEntry, List[PlatformDlcEntry], Dict[Any, PlatformDlcEntry]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "platform": "platform",
            "platformDlcIdMap": "platform_dlc_id_map",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "platform": False,
            "platformDlcIdMap": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "platform": ["EPICGAMES", "PSN", "STEAM", "XBOX"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Matchmaking Service (2.18.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_update_ruleset import ModelsUpdateRuleset


class ModelsUpdateChannelRequest(Model):
    """Models update channel request (models.UpdateChannelRequest)

    Properties:
        deployment: (deployment) REQUIRED str

        description: (description) REQUIRED str

        find_match_timeout_seconds: (findMatchTimeoutSeconds) REQUIRED int

        joinable: (joinable) REQUIRED bool

        max_delay_ms: (max_delay_ms) REQUIRED int

        rule_set: (ruleSet) REQUIRED ModelsUpdateRuleset

        session_queue_timeout_seconds: (sessionQueueTimeoutSeconds) REQUIRED int

        social_matchmaking: (socialMatchmaking) REQUIRED bool

        use_sub_gamemode: (use_sub_gamemode) REQUIRED bool
    """

    # region fields

    deployment: str  # REQUIRED
    description: str  # REQUIRED
    find_match_timeout_seconds: int  # REQUIRED
    joinable: bool  # REQUIRED
    max_delay_ms: int  # REQUIRED
    rule_set: ModelsUpdateRuleset  # REQUIRED
    session_queue_timeout_seconds: int  # REQUIRED
    social_matchmaking: bool  # REQUIRED
    use_sub_gamemode: bool  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_deployment(self, value: str) -> ModelsUpdateChannelRequest:
        self.deployment = value
        return self

    def with_description(self, value: str) -> ModelsUpdateChannelRequest:
        self.description = value
        return self

    def with_find_match_timeout_seconds(self, value: int) -> ModelsUpdateChannelRequest:
        self.find_match_timeout_seconds = value
        return self

    def with_joinable(self, value: bool) -> ModelsUpdateChannelRequest:
        self.joinable = value
        return self

    def with_max_delay_ms(self, value: int) -> ModelsUpdateChannelRequest:
        self.max_delay_ms = value
        return self

    def with_rule_set(self, value: ModelsUpdateRuleset) -> ModelsUpdateChannelRequest:
        self.rule_set = value
        return self

    def with_session_queue_timeout_seconds(
        self, value: int
    ) -> ModelsUpdateChannelRequest:
        self.session_queue_timeout_seconds = value
        return self

    def with_social_matchmaking(self, value: bool) -> ModelsUpdateChannelRequest:
        self.social_matchmaking = value
        return self

    def with_use_sub_gamemode(self, value: bool) -> ModelsUpdateChannelRequest:
        self.use_sub_gamemode = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "deployment"):
            result["deployment"] = str(self.deployment)
        elif include_empty:
            result["deployment"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "find_match_timeout_seconds"):
            result["findMatchTimeoutSeconds"] = int(self.find_match_timeout_seconds)
        elif include_empty:
            result["findMatchTimeoutSeconds"] = 0
        if hasattr(self, "joinable"):
            result["joinable"] = bool(self.joinable)
        elif include_empty:
            result["joinable"] = False
        if hasattr(self, "max_delay_ms"):
            result["max_delay_ms"] = int(self.max_delay_ms)
        elif include_empty:
            result["max_delay_ms"] = 0
        if hasattr(self, "rule_set"):
            result["ruleSet"] = self.rule_set.to_dict(include_empty=include_empty)
        elif include_empty:
            result["ruleSet"] = ModelsUpdateRuleset()
        if hasattr(self, "session_queue_timeout_seconds"):
            result["sessionQueueTimeoutSeconds"] = int(
                self.session_queue_timeout_seconds
            )
        elif include_empty:
            result["sessionQueueTimeoutSeconds"] = 0
        if hasattr(self, "social_matchmaking"):
            result["socialMatchmaking"] = bool(self.social_matchmaking)
        elif include_empty:
            result["socialMatchmaking"] = False
        if hasattr(self, "use_sub_gamemode"):
            result["use_sub_gamemode"] = bool(self.use_sub_gamemode)
        elif include_empty:
            result["use_sub_gamemode"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        deployment: str,
        description: str,
        find_match_timeout_seconds: int,
        joinable: bool,
        max_delay_ms: int,
        rule_set: ModelsUpdateRuleset,
        session_queue_timeout_seconds: int,
        social_matchmaking: bool,
        use_sub_gamemode: bool,
    ) -> ModelsUpdateChannelRequest:
        instance = cls()
        instance.deployment = deployment
        instance.description = description
        instance.find_match_timeout_seconds = find_match_timeout_seconds
        instance.joinable = joinable
        instance.max_delay_ms = max_delay_ms
        instance.rule_set = rule_set
        instance.session_queue_timeout_seconds = session_queue_timeout_seconds
        instance.social_matchmaking = social_matchmaking
        instance.use_sub_gamemode = use_sub_gamemode
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsUpdateChannelRequest:
        instance = cls()
        if not dict_:
            return instance
        if "deployment" in dict_ and dict_["deployment"] is not None:
            instance.deployment = str(dict_["deployment"])
        elif include_empty:
            instance.deployment = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if (
            "findMatchTimeoutSeconds" in dict_
            and dict_["findMatchTimeoutSeconds"] is not None
        ):
            instance.find_match_timeout_seconds = int(dict_["findMatchTimeoutSeconds"])
        elif include_empty:
            instance.find_match_timeout_seconds = 0
        if "joinable" in dict_ and dict_["joinable"] is not None:
            instance.joinable = bool(dict_["joinable"])
        elif include_empty:
            instance.joinable = False
        if "max_delay_ms" in dict_ and dict_["max_delay_ms"] is not None:
            instance.max_delay_ms = int(dict_["max_delay_ms"])
        elif include_empty:
            instance.max_delay_ms = 0
        if "ruleSet" in dict_ and dict_["ruleSet"] is not None:
            instance.rule_set = ModelsUpdateRuleset.create_from_dict(
                dict_["ruleSet"], include_empty=include_empty
            )
        elif include_empty:
            instance.rule_set = ModelsUpdateRuleset()
        if (
            "sessionQueueTimeoutSeconds" in dict_
            and dict_["sessionQueueTimeoutSeconds"] is not None
        ):
            instance.session_queue_timeout_seconds = int(
                dict_["sessionQueueTimeoutSeconds"]
            )
        elif include_empty:
            instance.session_queue_timeout_seconds = 0
        if "socialMatchmaking" in dict_ and dict_["socialMatchmaking"] is not None:
            instance.social_matchmaking = bool(dict_["socialMatchmaking"])
        elif include_empty:
            instance.social_matchmaking = False
        if "use_sub_gamemode" in dict_ and dict_["use_sub_gamemode"] is not None:
            instance.use_sub_gamemode = bool(dict_["use_sub_gamemode"])
        elif include_empty:
            instance.use_sub_gamemode = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsUpdateChannelRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsUpdateChannelRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsUpdateChannelRequest,
        List[ModelsUpdateChannelRequest],
        Dict[Any, ModelsUpdateChannelRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "deployment": "deployment",
            "description": "description",
            "findMatchTimeoutSeconds": "find_match_timeout_seconds",
            "joinable": "joinable",
            "max_delay_ms": "max_delay_ms",
            "ruleSet": "rule_set",
            "sessionQueueTimeoutSeconds": "session_queue_timeout_seconds",
            "socialMatchmaking": "social_matchmaking",
            "use_sub_gamemode": "use_sub_gamemode",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "deployment": True,
            "description": True,
            "findMatchTimeoutSeconds": True,
            "joinable": True,
            "max_delay_ms": True,
            "ruleSet": True,
            "sessionQueueTimeoutSeconds": True,
            "socialMatchmaking": True,
            "use_sub_gamemode": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Matchmaking Service (2.18.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_alliance_flexing_rule import ModelsAllianceFlexingRule
from ..models.models_alliance_rule import ModelsAllianceRule
from ..models.models_flexing_rule import ModelsFlexingRule
from ..models.models_match_option_rule import ModelsMatchOptionRule
from ..models.models_matching_rule import ModelsMatchingRule
from ..models.models_sub_game_mode import ModelsSubGameMode


class ModelsRuleSet(Model):
    """Models rule set (models.RuleSet)

    Properties:
        alliance: (alliance) REQUIRED ModelsAllianceRule

        alliance_flexing_rule: (alliance_flexing_rule) REQUIRED List[ModelsAllianceFlexingRule]

        flexing_rule: (flexing_rule) REQUIRED List[ModelsFlexingRule]

        match_options: (match_options) REQUIRED ModelsMatchOptionRule

        matching_rule: (matching_rule) REQUIRED List[ModelsMatchingRule]

        rebalance_enable: (rebalance_enable) REQUIRED bool

        sub_game_modes: (sub_game_modes) REQUIRED Dict[str, ModelsSubGameMode]
    """

    # region fields

    alliance: ModelsAllianceRule  # REQUIRED
    alliance_flexing_rule: List[ModelsAllianceFlexingRule]  # REQUIRED
    flexing_rule: List[ModelsFlexingRule]  # REQUIRED
    match_options: ModelsMatchOptionRule  # REQUIRED
    matching_rule: List[ModelsMatchingRule]  # REQUIRED
    rebalance_enable: bool  # REQUIRED
    sub_game_modes: Dict[str, ModelsSubGameMode]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_alliance(self, value: ModelsAllianceRule) -> ModelsRuleSet:
        self.alliance = value
        return self

    def with_alliance_flexing_rule(
        self, value: List[ModelsAllianceFlexingRule]
    ) -> ModelsRuleSet:
        self.alliance_flexing_rule = value
        return self

    def with_flexing_rule(self, value: List[ModelsFlexingRule]) -> ModelsRuleSet:
        self.flexing_rule = value
        return self

    def with_match_options(self, value: ModelsMatchOptionRule) -> ModelsRuleSet:
        self.match_options = value
        return self

    def with_matching_rule(self, value: List[ModelsMatchingRule]) -> ModelsRuleSet:
        self.matching_rule = value
        return self

    def with_rebalance_enable(self, value: bool) -> ModelsRuleSet:
        self.rebalance_enable = value
        return self

    def with_sub_game_modes(self, value: Dict[str, ModelsSubGameMode]) -> ModelsRuleSet:
        self.sub_game_modes = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "alliance"):
            result["alliance"] = self.alliance.to_dict(include_empty=include_empty)
        elif include_empty:
            result["alliance"] = ModelsAllianceRule()
        if hasattr(self, "alliance_flexing_rule"):
            result["alliance_flexing_rule"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.alliance_flexing_rule
            ]
        elif include_empty:
            result["alliance_flexing_rule"] = []
        if hasattr(self, "flexing_rule"):
            result["flexing_rule"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.flexing_rule
            ]
        elif include_empty:
            result["flexing_rule"] = []
        if hasattr(self, "match_options"):
            result["match_options"] = self.match_options.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["match_options"] = ModelsMatchOptionRule()
        if hasattr(self, "matching_rule"):
            result["matching_rule"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.matching_rule
            ]
        elif include_empty:
            result["matching_rule"] = []
        if hasattr(self, "rebalance_enable"):
            result["rebalance_enable"] = bool(self.rebalance_enable)
        elif include_empty:
            result["rebalance_enable"] = False
        if hasattr(self, "sub_game_modes"):
            result["sub_game_modes"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.sub_game_modes.items()
            }
        elif include_empty:
            result["sub_game_modes"] = {}
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        alliance: ModelsAllianceRule,
        alliance_flexing_rule: List[ModelsAllianceFlexingRule],
        flexing_rule: List[ModelsFlexingRule],
        match_options: ModelsMatchOptionRule,
        matching_rule: List[ModelsMatchingRule],
        rebalance_enable: bool,
        sub_game_modes: Dict[str, ModelsSubGameMode],
    ) -> ModelsRuleSet:
        instance = cls()
        instance.alliance = alliance
        instance.alliance_flexing_rule = alliance_flexing_rule
        instance.flexing_rule = flexing_rule
        instance.match_options = match_options
        instance.matching_rule = matching_rule
        instance.rebalance_enable = rebalance_enable
        instance.sub_game_modes = sub_game_modes
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsRuleSet:
        instance = cls()
        if not dict_:
            return instance
        if "alliance" in dict_ and dict_["alliance"] is not None:
            instance.alliance = ModelsAllianceRule.create_from_dict(
                dict_["alliance"], include_empty=include_empty
            )
        elif include_empty:
            instance.alliance = ModelsAllianceRule()
        if (
            "alliance_flexing_rule" in dict_
            and dict_["alliance_flexing_rule"] is not None
        ):
            instance.alliance_flexing_rule = [
                ModelsAllianceFlexingRule.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["alliance_flexing_rule"]
            ]
        elif include_empty:
            instance.alliance_flexing_rule = []
        if "flexing_rule" in dict_ and dict_["flexing_rule"] is not None:
            instance.flexing_rule = [
                ModelsFlexingRule.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["flexing_rule"]
            ]
        elif include_empty:
            instance.flexing_rule = []
        if "match_options" in dict_ and dict_["match_options"] is not None:
            instance.match_options = ModelsMatchOptionRule.create_from_dict(
                dict_["match_options"], include_empty=include_empty
            )
        elif include_empty:
            instance.match_options = ModelsMatchOptionRule()
        if "matching_rule" in dict_ and dict_["matching_rule"] is not None:
            instance.matching_rule = [
                ModelsMatchingRule.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["matching_rule"]
            ]
        elif include_empty:
            instance.matching_rule = []
        if "rebalance_enable" in dict_ and dict_["rebalance_enable"] is not None:
            instance.rebalance_enable = bool(dict_["rebalance_enable"])
        elif include_empty:
            instance.rebalance_enable = False
        if "sub_game_modes" in dict_ and dict_["sub_game_modes"] is not None:
            instance.sub_game_modes = {
                str(k0): ModelsSubGameMode.create_from_dict(
                    v0, include_empty=include_empty
                )
                for k0, v0 in dict_["sub_game_modes"].items()
            }
        elif include_empty:
            instance.sub_game_modes = {}
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsRuleSet]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsRuleSet]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ModelsRuleSet, List[ModelsRuleSet], Dict[Any, ModelsRuleSet]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "alliance": "alliance",
            "alliance_flexing_rule": "alliance_flexing_rule",
            "flexing_rule": "flexing_rule",
            "match_options": "match_options",
            "matching_rule": "matching_rule",
            "rebalance_enable": "rebalance_enable",
            "sub_game_modes": "sub_game_modes",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "alliance": True,
            "alliance_flexing_rule": True,
            "flexing_rule": True,
            "match_options": True,
            "matching_rule": True,
            "rebalance_enable": True,
            "sub_game_modes": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Matchmaking Service (2.18.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.matchmaking.models import LogAppMessageDeclaration
from ..api.matchmaking.models import ModelsAllianceFlexingRule
from ..api.matchmaking.models import ModelsAllianceRule
from ..api.matchmaking.models import ModelsAllianceRuleV1
from ..api.matchmaking.models import ModelsChannel
from ..api.matchmaking.models import ModelsChannelRequest
from ..api.matchmaking.models import ModelsChannelV1
from ..api.matchmaking.models import ModelsCombination
from ..api.matchmaking.models import ModelsCombinationAlliances
from ..api.matchmaking.models import ModelsCreateChannelResponse
from ..api.matchmaking.models import ModelsDequeueRequest
from ..api.matchmaking.models import ModelsFlexingRule
from ..api.matchmaking.models import ModelsGetChannelsResponse
from ..api.matchmaking.models import ModelsImportConfigResponse
from ..api.matchmaking.models import ModelsMatchAddUserIntoSessionRequest
from ..api.matchmaking.models import ModelsMatchOption
from ..api.matchmaking.models import ModelsMatchOptionRule
from ..api.matchmaking.models import ModelsMatchResultRequest
from ..api.matchmaking.models import ModelsMatchResultResponse
from ..api.matchmaking.models import ModelsMatchingAlly
from ..api.matchmaking.models import ModelsMatchingParty
from ..api.matchmaking.models import ModelsMatchingRule
from ..api.matchmaking.models import ModelsMatchmakingResult
from ..api.matchmaking.models import ModelsPagination
from ..api.matchmaking.models import ModelsPartyMember
from ..api.matchmaking.models import ModelsPlayerResultRequest
from ..api.matchmaking.models import ModelsPlayerResultResponse
from ..api.matchmaking.models import ModelsRebalanceRequest
from ..api.matchmaking.models import ModelsRebalanceResponse
from ..api.matchmaking.models import ModelsResultAttributeRequest
from ..api.matchmaking.models import ModelsResultAttributeResponse
from ..api.matchmaking.models import ModelsRuleSet
from ..api.matchmaking.models import ModelsRuleSetV1
from ..api.matchmaking.models import ModelsSubGameMode
from ..api.matchmaking.models import ModelsUpdateAllianceRule
from ..api.matchmaking.models import ModelsUpdateChannelRequest
from ..api.matchmaking.models import ModelsUpdatePlayTimeWeightRequest
from ..api.matchmaking.models import ModelsUpdatePlayerPlaytimeWeightResponse
from ..api.matchmaking.models import ModelsUpdateRuleset
from ..api.matchmaking.models import ModelsUpdateRulesetSubGameModes
from ..api.matchmaking.models import ResponseError
from ..api.matchmaking.models import ResponseErrorV1
from ..api.matchmaking.models import ServiceGetSessionHistoryDetailedResponseItem
from ..api.matchmaking.models import ServiceGetSessionHistorySearchResponse
from ..api.matchmaking.models import ServiceGetSessionHistorySearchResponseItem
from ..api.matchmaking.models import ServiceGetSessionHistorySearchResponseItemV2
from ..api.matchmaking.models import ServiceGetSessionHistorySearchResponseV2


def create_log_app_message_declaration_example() -> LogAppMessageDeclaration:
    instance = LogAppMessageDeclaration()
    instance.attributes = [randomize()]
    instance.code = randomize()
    instance.code_name = randomize()
    instance.section = randomize()
    instance.service = randomize()
    instance.text = randomize()
    return instance


def create_models_alliance_flexing_rule_example() -> ModelsAllianceFlexingRule:
    instance = ModelsAllianceFlexingRule()
    instance.combination = create_models_combination_example()
    instance.duration = randomize("int", min_val=1, max_val=1000)
    instance.max_number = randomize("int", min_val=1, max_val=1000)
    instance.min_number = randomize("int", min_val=1, max_val=1000)
    instance.player_max_number = randomize("int", min_val=1, max_val=1000)
    instance.player_min_number = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_alliance_rule_example() -> ModelsAllianceRule:
    instance = ModelsAllianceRule()
    instance.combination = create_models_combination_example()
    instance.max_number = randomize("int", min_val=1, max_val=1000)
    instance.min_number = randomize("int", min_val=1, max_val=1000)
    instance.player_max_number = randomize("int", min_val=1, max_val=1000)
    instance.player_min_number = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_alliance_rule_v1_example() -> ModelsAllianceRuleV1:
    instance = ModelsAllianceRuleV1()
    instance.combination = create_models_combination_example()
    instance.max_number = randomize("int", min_val=1, max_val=1000)
    instance.min_number = randomize("int", min_val=1, max_val=1000)
    instance.player_max_number = randomize("int", min_val=1, max_val=1000)
    instance.player_min_number = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_channel_example() -> ModelsChannel:
    instance = ModelsChannel()
    instance.deployment = randomize()
    instance.description = randomize()
    instance.find_match_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.game_mode = randomize()
    instance.joinable = randomize("bool")
    instance.max_delay_ms = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.ruleset = create_models_rule_set_example()
    instance.session_queue_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.slug = randomize()
    instance.social_matchmaking = randomize("bool")
    instance.updated_at = randomize()
    instance.use_sub_gamemode = randomize("bool")
    return instance


def create_models_channel_request_example() -> ModelsChannelRequest:
    instance = ModelsChannelRequest()
    instance.deployment = randomize()
    instance.description = randomize()
    instance.find_match_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.game_mode = randomize()
    instance.max_delay_ms = randomize("int", min_val=1, max_val=1000)
    instance.rule_set = create_models_rule_set_example()
    instance.session_queue_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.joinable = randomize("bool")
    instance.social_matchmaking = randomize("bool")
    instance.use_sub_gamemode = randomize("bool")
    return instance


def create_models_channel_v1_example() -> ModelsChannelV1:
    instance = ModelsChannelV1()
    instance.deployment = randomize()
    instance.description = randomize()
    instance.find_match_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.game_mode = randomize()
    instance.joinable = randomize("bool")
    instance.max_delay_ms = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.rule_set = create_models_rule_set_v1_example()
    instance.session_queue_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.slug = randomize()
    instance.social_matchmaking = randomize("bool")
    instance.updated_at = randomize()
    instance.use_sub_gamemode = randomize("bool")
    return instance


def create_models_combination_example() -> ModelsCombination:
    instance = ModelsCombination()
    instance.alliances = [create_models_combination_alliances_example()]
    instance.has_combination = randomize("bool")
    instance.role_flexing_enable = randomize("bool")
    instance.role_flexing_player = randomize("int", min_val=1, max_val=1000)
    instance.role_flexing_second = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_combination_alliances_example() -> ModelsCombinationAlliances:
    instance = ModelsCombinationAlliances()
    instance.max = randomize("int", min_val=1, max_val=1000)
    instance.min = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    return instance


def create_models_create_channel_response_example() -> ModelsCreateChannelResponse:
    instance = ModelsCreateChannelResponse()
    instance.deployment = randomize()
    instance.description = randomize()
    instance.find_match_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.game_mode = randomize()
    instance.joinable = randomize("bool")
    instance.max_delay_ms = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.ruleset = create_models_rule_set_example()
    instance.session_queue_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.slug = randomize()
    instance.social_matchmaking = randomize("bool")
    instance.updated_at = randomize()
    instance.use_sub_gamemode = randomize("bool")
    return instance


def create_models_dequeue_request_example() -> ModelsDequeueRequest:
    instance = ModelsDequeueRequest()
    instance.match_id = randomize()
    return instance


def create_models_flexing_rule_example() -> ModelsFlexingRule:
    instance = ModelsFlexingRule()
    instance.attribute = randomize()
    instance.criteria = randomize()
    instance.duration = randomize("int", min_val=1, max_val=1000)
    instance.reference = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_get_channels_response_example() -> ModelsGetChannelsResponse:
    instance = ModelsGetChannelsResponse()
    instance.data = [create_models_channel_example()]
    instance.pagination = create_models_pagination_example()
    return instance


def create_models_import_config_response_example() -> ModelsImportConfigResponse:
    instance = ModelsImportConfigResponse()
    instance.failed_configs = [randomize()]
    instance.ignored_configs = [randomize()]
    instance.new_configs = [randomize()]
    instance.replaced_configs = [randomize()]
    return instance


def create_models_match_add_user_into_session_request_example() -> ModelsMatchAddUserIntoSessionRequest:
    instance = ModelsMatchAddUserIntoSessionRequest()
    instance.user_id = randomize("uid")
    instance.blocked_players = [randomize()]
    instance.party_id = randomize("uid")
    return instance


def create_models_match_option_example() -> ModelsMatchOption:
    instance = ModelsMatchOption()
    instance.name = randomize()
    instance.type_ = randomize()
    return instance


def create_models_match_option_rule_example() -> ModelsMatchOptionRule:
    instance = ModelsMatchOptionRule()
    instance.options = [create_models_match_option_example()]
    return instance


def create_models_match_result_request_example() -> ModelsMatchResultRequest:
    instance = ModelsMatchResultRequest()
    instance.match_id = randomize()
    instance.players = [create_models_player_result_request_example()]
    return instance


def create_models_match_result_response_example() -> ModelsMatchResultResponse:
    instance = ModelsMatchResultResponse()
    instance.match_id = randomize()
    instance.players = [create_models_player_result_response_example()]
    return instance


def create_models_matching_ally_example() -> ModelsMatchingAlly:
    instance = ModelsMatchingAlly()
    instance.matching_parties = [create_models_matching_party_example()]
    return instance


def create_models_matching_party_example() -> ModelsMatchingParty:
    instance = ModelsMatchingParty()
    instance.first_ticket_created_at = randomize("int", min_val=1, max_val=1000)
    instance.party_attributes = {randomize(): randomize()}
    instance.party_id = randomize("uid")
    instance.party_members = [create_models_party_member_example()]
    instance.ticket_id = randomize()
    return instance


def create_models_matching_rule_example() -> ModelsMatchingRule:
    instance = ModelsMatchingRule()
    instance.attribute = randomize()
    instance.criteria = randomize()
    instance.reference = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_matchmaking_result_example() -> ModelsMatchmakingResult:
    instance = ModelsMatchmakingResult()
    instance.channel = randomize()
    instance.client_version = randomize()
    instance.deployment = randomize()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    instance.game_mode = randomize()
    instance.match_id = randomize()
    instance.matching_allies = [create_models_matching_ally_example()]
    instance.namespace = randomize("slug")
    instance.party_attributes = {randomize(): randomize()}
    instance.queued_at = randomize("int", min_val=1, max_val=1000)
    instance.region = randomize()
    instance.server_name = randomize()
    instance.status = randomize()
    instance.ticket_id = randomize()
    instance.updated_at = randomize("date")
    instance.joinable = randomize("bool")
    instance.party_id = randomize("uid")
    return instance


def create_models_pagination_example() -> ModelsPagination:
    instance = ModelsPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_party_member_example() -> ModelsPartyMember:
    instance = ModelsPartyMember()
    instance.extra_attributes = {randomize(): randomize()}
    instance.user_id = randomize("uid")
    return instance


def create_models_player_result_request_example() -> ModelsPlayerResultRequest:
    instance = ModelsPlayerResultRequest()
    instance.results = [create_models_result_attribute_request_example()]
    instance.user_id = randomize("uid")
    return instance


def create_models_player_result_response_example() -> ModelsPlayerResultResponse:
    instance = ModelsPlayerResultResponse()
    instance.current_score = [create_models_result_attribute_response_example()]
    instance.user_id = randomize("uid")
    return instance


def create_models_rebalance_request_example() -> ModelsRebalanceRequest:
    instance = ModelsRebalanceRequest()
    instance.match_id = randomize()
    return instance


def create_models_rebalance_response_example() -> ModelsRebalanceResponse:
    instance = ModelsRebalanceResponse()
    instance.match_id = randomize()
    instance.matching_allies = [create_models_matching_ally_example()]
    return instance


def create_models_result_attribute_request_example() -> ModelsResultAttributeRequest:
    instance = ModelsResultAttributeRequest()
    instance.attribute = randomize()
    instance.value = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_result_attribute_response_example() -> ModelsResultAttributeResponse:
    instance = ModelsResultAttributeResponse()
    instance.attribute = randomize()
    instance.value = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_rule_set_example() -> ModelsRuleSet:
    instance = ModelsRuleSet()
    instance.alliance = create_models_alliance_rule_example()
    instance.alliance_flexing_rule = [create_models_alliance_flexing_rule_example()]
    instance.flexing_rule = [create_models_flexing_rule_example()]
    instance.match_options = create_models_match_option_rule_example()
    instance.matching_rule = [create_models_matching_rule_example()]
    instance.rebalance_enable = randomize("bool")
    instance.sub_game_modes = {}
    return instance


def create_models_rule_set_v1_example() -> ModelsRuleSetV1:
    instance = ModelsRuleSetV1()
    instance.alliance = create_models_alliance_rule_v1_example()
    instance.alliance_flexing_rule = [create_models_alliance_flexing_rule_example()]
    instance.flexing_rules = [create_models_flexing_rule_example()]
    instance.match_options = create_models_match_option_rule_example()
    instance.matching_rules = [create_models_matching_rule_example()]
    instance.sub_game_modes = {}
    return instance


def create_models_sub_game_mode_example() -> ModelsSubGameMode:
    instance = ModelsSubGameMode()
    instance.alliance = create_models_alliance_rule_example()
    instance.alliance_flexing_rule = [create_models_alliance_flexing_rule_example()]
    instance.name = randomize()
    return instance


def create_models_update_alliance_rule_example() -> ModelsUpdateAllianceRule:
    instance = ModelsUpdateAllianceRule()
    instance.combination = create_models_combination_example()
    instance.max_number = randomize("int", min_val=1, max_val=1000)
    instance.min_number = randomize("int", min_val=1, max_val=1000)
    instance.player_max_number = randomize("int", min_val=1, max_val=1000)
    instance.player_min_number = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_channel_request_example() -> ModelsUpdateChannelRequest:
    instance = ModelsUpdateChannelRequest()
    instance.deployment = randomize()
    instance.description = randomize()
    instance.find_match_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.joinable = randomize("bool")
    instance.max_delay_ms = randomize("int", min_val=1, max_val=1000)
    instance.rule_set = create_models_update_ruleset_example()
    instance.session_queue_timeout_seconds = randomize("int", min_val=1, max_val=1000)
    instance.social_matchmaking = randomize("bool")
    instance.use_sub_gamemode = randomize("bool")
    return instance


def create_models_update_play_time_weight_request_example() -> ModelsUpdatePlayTimeWeightRequest:
    instance = ModelsUpdatePlayTimeWeightRequest()
    instance.playtime = randomize("int", min_val=1, max_val=1000)
    instance.user_id = randomize("uid")
    instance.weight = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_player_playtime_weight_response_example() -> ModelsUpdatePlayerPlaytimeWeightResponse:
    instance = ModelsUpdatePlayerPlaytimeWeightResponse()
    instance.playtime = randomize()
    instance.user_id = randomize("uid")
    instance.weight = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_ruleset_example() -> ModelsUpdateRuleset:
    instance = ModelsUpdateRuleset()
    instance.alliance = create_models_update_alliance_rule_example()
    instance.alliance_flexing_rule = [create_models_alliance_flexing_rule_example()]
    instance.flexing_rules = [create_models_flexing_rule_example()]
    instance.match_options = create_models_match_option_rule_example()
    instance.matching_rules = [create_models_matching_rule_example()]
    instance.sub_game_modes = create_models_update_ruleset_sub_game_modes_example()
    return instance


def create_models_update_ruleset_sub_game_modes_example() -> ModelsUpdateRulesetSubGameModes:
    instance = ModelsUpdateRulesetSubGameModes()
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance


def create_response_error_v1_example() -> ResponseErrorV1:
    instance = ResponseErrorV1()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance


def create_service_get_session_history_detailed_response_item_example() -> ServiceGetSessionHistoryDetailedResponseItem:
    instance = ServiceGetSessionHistoryDetailedResponseItem()
    instance.channel = randomize()
    instance.client_version = randomize()
    instance.created_at = randomize("date")
    instance.event_description = randomize()
    instance.event_name = randomize()
    instance.game_mode = randomize()
    instance.id_ = randomize()
    instance.joinable = randomize("bool")
    instance.match_id = randomize()
    instance.matching_allies = [create_models_matching_ally_example()]
    instance.namespace = randomize("slug")
    instance.region = randomize()
    instance.server_name = randomize()
    instance.status = randomize()
    instance.party_id = randomize("uid")
    return instance


def create_service_get_session_history_search_response_example() -> ServiceGetSessionHistorySearchResponse:
    instance = ServiceGetSessionHistorySearchResponse()
    instance.data = [create_service_get_session_history_search_response_item_example()]
    instance.pagination = create_models_pagination_example()
    return instance


def create_service_get_session_history_search_response_item_example() -> ServiceGetSessionHistorySearchResponseItem:
    instance = ServiceGetSessionHistorySearchResponseItem()
    instance.channel = randomize()
    instance.client_version = randomize()
    instance.created_at = randomize("date")
    instance.event_description = randomize()
    instance.event_name = randomize()
    instance.game_mode = randomize()
    instance.id_ = randomize()
    instance.joinable = randomize("bool")
    instance.match_id = randomize()
    instance.namespace = randomize("slug")
    instance.region = randomize()
    instance.server_name = randomize()
    instance.status = randomize()
    instance.sub_game_mode = [randomize()]
    instance.party_id = randomize("uid")
    return instance


def create_service_get_session_history_search_response_item_v2_example() -> ServiceGetSessionHistorySearchResponseItemV2:
    instance = ServiceGetSessionHistorySearchResponseItemV2()
    instance.created_at = randomize("date")
    instance.game_mode = randomize()
    instance.id_ = randomize()
    instance.joinable = randomize("bool")
    instance.match_id = randomize()
    instance.namespace = randomize("slug")
    instance.status = randomize()
    instance.sub_game_mode = [randomize()]
    return instance


def create_service_get_session_history_search_response_v2_example() -> ServiceGetSessionHistorySearchResponseV2:
    instance = ServiceGetSessionHistorySearchResponseV2()
    instance.data = [
        create_service_get_session_history_search_response_item_v2_example()
    ]
    instance.pagination = create_models_pagination_example()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Social Service."""

__version__ = "1.31.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_stat import CreateStat
from .create_stat_1 import CreateStat1
from .delete_stat import DeleteStat
from .delete_tied_stat import DeleteTiedStat
from .export_stats import ExportStats
from .get_stat import GetStat
from .get_stats import GetStats
from .import_stats import ImportStats
from .query_stats import QueryStats
from .update_stat import UpdateStat
